import { isPromise } from "./jsutils/isPromise.mjs";

import { validateSchema } from "./type/validate.mjs";

import { validate } from "./validation/validate.mjs";

import { execute } from "./execution/execute.mjs";

import { parse } from "./shared.mjs";

function graphql(args) {
  return new Promise((resolve => resolve(graphqlImpl(args))));
}

function graphqlSync(args) {
  const result = graphqlImpl(args);
  if (isPromise(result)) {
    throw new Error("GraphQL execution failed to complete synchronously.");
  }
  return result;
}

function graphqlImpl(args) {
  const {schema, source, rootValue, contextValue, variableValues, operationName, fieldResolver, typeResolver} = args;
  const schemaValidationErrors = validateSchema(schema);
  if (schemaValidationErrors.length > 0) {
    return {
      errors: schemaValidationErrors
    };
  }
  let document;
  try {
    document = parse(source);
  } catch (syntaxError) {
    return {
      errors: [ syntaxError ]
    };
  }
  const validationErrors = validate(schema, document);
  if (validationErrors.length > 0) {
    return {
      errors: validationErrors
    };
  }
  return execute({
    schema,
    document,
    rootValue,
    contextValue,
    variableValues,
    operationName,
    fieldResolver,
    typeResolver
  });
}

export { graphql, graphqlSync };
