import { e } from "../shared.mjs";

function isDefinitionNode(node) {
  return isExecutableDefinitionNode(node) || isTypeSystemDefinitionNode(node) || isTypeSystemExtensionNode(node);
}

function isExecutableDefinitionNode(node) {
  return node.kind === e.OPERATION_DEFINITION || node.kind === e.FRAGMENT_DEFINITION;
}

function isSelectionNode(node) {
  return node.kind === e.FIELD || node.kind === e.FRAGMENT_SPREAD || node.kind === e.INLINE_FRAGMENT;
}

function isValueNode(node) {
  return node.kind === e.VARIABLE || node.kind === e.INT || node.kind === e.FLOAT || node.kind === e.STRING || node.kind === e.BOOLEAN || node.kind === e.NULL || node.kind === e.ENUM || node.kind === e.LIST || node.kind === e.OBJECT;
}

function isTypeNode(node) {
  return node.kind === e.NAMED_TYPE || node.kind === e.LIST_TYPE || node.kind === e.NON_NULL_TYPE;
}

function isTypeSystemDefinitionNode(node) {
  return node.kind === e.SCHEMA_DEFINITION || isTypeDefinitionNode(node) || node.kind === e.DIRECTIVE_DEFINITION;
}

function isTypeDefinitionNode(node) {
  return node.kind === e.SCALAR_TYPE_DEFINITION || node.kind === e.OBJECT_TYPE_DEFINITION || node.kind === e.INTERFACE_TYPE_DEFINITION || node.kind === e.UNION_TYPE_DEFINITION || node.kind === e.ENUM_TYPE_DEFINITION || node.kind === e.INPUT_OBJECT_TYPE_DEFINITION;
}

function isTypeSystemExtensionNode(node) {
  return node.kind === e.SCHEMA_EXTENSION || isTypeExtensionNode(node);
}

function isTypeExtensionNode(node) {
  return node.kind === e.SCALAR_TYPE_EXTENSION || node.kind === e.OBJECT_TYPE_EXTENSION || node.kind === e.INTERFACE_TYPE_EXTENSION || node.kind === e.UNION_TYPE_EXTENSION || node.kind === e.ENUM_TYPE_EXTENSION || node.kind === e.INPUT_OBJECT_TYPE_EXTENSION;
}

export { isDefinitionNode, isExecutableDefinitionNode, isSelectionNode, isTypeDefinitionNode, isTypeExtensionNode, isTypeNode, isTypeSystemDefinitionNode, isTypeSystemExtensionNode, isValueNode };
