var jsutils_didYouMean = require("../jsutils/didYouMean.js");

var jsutils_identityFunc = require("../jsutils/identityFunc.js");

var jsutils_inspect = require("../jsutils/inspect.js");

var jsutils_instanceOf = require("../jsutils/instanceOf.js");

var jsutils_keyMap = require("../jsutils/keyMap.js");

var jsutils_keyValMap = require("../jsutils/keyValMap.js");

var jsutils_mapValue = require("../jsutils/mapValue.js");

var jsutils_suggestionList = require("../jsutils/suggestionList.js");

var jsutils_toObjMap = require("../jsutils/toObjMap.js");

var utilities_valueFromASTUntyped = require("../utilities/valueFromASTUntyped.js");

var type_assertName = require("./assertName.js");

var shared = require("../shared.js");

function isType(type) {
  return isScalarType(type) || isObjectType(type) || isInterfaceType(type) || isUnionType(type) || isEnumType(type) || isInputObjectType(type) || isListType(type) || isNonNullType(type);
}

function isScalarType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLScalarType);
}

function isObjectType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLObjectType);
}

function isInterfaceType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLInterfaceType);
}

function isUnionType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLUnionType);
}

function isEnumType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLEnumType);
}

function isInputObjectType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLInputObjectType);
}

function isListType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLList);
}

function isNonNullType(type) {
  return jsutils_instanceOf.instanceOf(type, GraphQLNonNull);
}

function isInputType(type) {
  return isScalarType(type) || isEnumType(type) || isInputObjectType(type) || isWrappingType(type) && isInputType(type.ofType);
}

function isOutputType(type) {
  return isScalarType(type) || isObjectType(type) || isInterfaceType(type) || isUnionType(type) || isEnumType(type) || isWrappingType(type) && isOutputType(type.ofType);
}

function isLeafType(type) {
  return isScalarType(type) || isEnumType(type);
}

function isCompositeType(type) {
  return isObjectType(type) || isInterfaceType(type) || isUnionType(type);
}

function isAbstractType(type) {
  return isInterfaceType(type) || isUnionType(type);
}

class GraphQLList {
  constructor(ofType) {
    isType(ofType);
    this.ofType = ofType;
  }
  toString() {
    return "[" + String(this.ofType) + "]";
  }
  toJSON() {
    return this.toString();
  }
}

class GraphQLNonNull {
  constructor(ofType) {
    isNullableType(ofType);
    this.ofType = ofType;
  }
  toString() {
    return String(this.ofType) + "!";
  }
  toJSON() {
    return this.toString();
  }
}

function isWrappingType(type) {
  return isListType(type) || isNonNullType(type);
}

function isNullableType(type) {
  return isType(type) && !isNonNullType(type);
}

function isNamedType(type) {
  return isScalarType(type) || isObjectType(type) || isInterfaceType(type) || isUnionType(type) || isEnumType(type) || isInputObjectType(type);
}

function resolveReadonlyArrayThunk(thunk) {
  return "function" == typeof thunk ? thunk() : thunk;
}

function resolveObjMapThunk(thunk) {
  return "function" == typeof thunk ? thunk() : thunk;
}

class GraphQLScalarType {
  constructor(config) {
    var _config$parseValue, _config$serialize, _config$parseLiteral, _config$extensionASTN;
    const parseValue = null !== (_config$parseValue = config.parseValue) && void 0 !== _config$parseValue ? _config$parseValue : jsutils_identityFunc.identityFunc;
    this.name = type_assertName.assertName(config.name);
    this.description = config.description;
    this.specifiedByURL = config.specifiedByURL;
    this.serialize = null !== (_config$serialize = config.serialize) && void 0 !== _config$serialize ? _config$serialize : jsutils_identityFunc.identityFunc;
    this.parseValue = parseValue;
    this.parseLiteral = null !== (_config$parseLiteral = config.parseLiteral) && void 0 !== _config$parseLiteral ? _config$parseLiteral : (node, variables) => parseValue(utilities_valueFromASTUntyped.valueFromASTUntyped(node, variables));
    this.extensions = jsutils_toObjMap.toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = null !== (_config$extensionASTN = config.extensionASTNodes) && void 0 !== _config$extensionASTN ? _config$extensionASTN : [];
  }
  toConfig() {
    return {
      name: this.name,
      description: this.description,
      specifiedByURL: this.specifiedByURL,
      serialize: this.serialize,
      parseValue: this.parseValue,
      parseLiteral: this.parseLiteral,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes
    };
  }
  toString() {
    return this.name;
  }
  toJSON() {
    return this.toString();
  }
}

class GraphQLObjectType {
  constructor(config) {
    var _config$extensionASTN2;
    this.name = type_assertName.assertName(config.name);
    this.description = config.description;
    this.isTypeOf = config.isTypeOf;
    this.extensions = jsutils_toObjMap.toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = null !== (_config$extensionASTN2 = config.extensionASTNodes) && void 0 !== _config$extensionASTN2 ? _config$extensionASTN2 : [];
    this._fields = () => defineFieldMap(config);
    this._interfaces = () => defineInterfaces(config);
  }
  getFields() {
    if ("function" == typeof this._fields) {
      this._fields = this._fields();
    }
    return this._fields;
  }
  getInterfaces() {
    if ("function" == typeof this._interfaces) {
      this._interfaces = this._interfaces();
    }
    return this._interfaces;
  }
  toConfig() {
    return {
      name: this.name,
      description: this.description,
      interfaces: this.getInterfaces(),
      fields: fieldsToFieldsConfig(this.getFields()),
      isTypeOf: this.isTypeOf,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes
    };
  }
  toString() {
    return this.name;
  }
  toJSON() {
    return this.toString();
  }
}

function defineInterfaces(config) {
  var _config$interfaces;
  return resolveReadonlyArrayThunk(null !== (_config$interfaces = config.interfaces) && void 0 !== _config$interfaces ? _config$interfaces : []);
}

function defineFieldMap(config) {
  const fieldMap = resolveObjMapThunk(config.fields);
  return jsutils_mapValue.mapValue(fieldMap, ((fieldConfig, fieldName) => {
    var _fieldConfig$args;
    const argsConfig = null !== (_fieldConfig$args = fieldConfig.args) && void 0 !== _fieldConfig$args ? _fieldConfig$args : {};
    return {
      name: type_assertName.assertName(fieldName),
      description: fieldConfig.description,
      type: fieldConfig.type,
      args: defineArguments(argsConfig),
      resolve: fieldConfig.resolve,
      subscribe: fieldConfig.subscribe,
      deprecationReason: fieldConfig.deprecationReason,
      extensions: jsutils_toObjMap.toObjMap(fieldConfig.extensions),
      astNode: fieldConfig.astNode
    };
  }));
}

function defineArguments(config) {
  return Object.entries(config).map((([argName, argConfig]) => ({
    name: type_assertName.assertName(argName),
    description: argConfig.description,
    type: argConfig.type,
    defaultValue: argConfig.defaultValue,
    deprecationReason: argConfig.deprecationReason,
    extensions: jsutils_toObjMap.toObjMap(argConfig.extensions),
    astNode: argConfig.astNode
  })));
}

function fieldsToFieldsConfig(fields) {
  return jsutils_mapValue.mapValue(fields, (field => ({
    description: field.description,
    type: field.type,
    args: argsToArgsConfig(field.args),
    resolve: field.resolve,
    subscribe: field.subscribe,
    deprecationReason: field.deprecationReason,
    extensions: field.extensions,
    astNode: field.astNode
  })));
}

function argsToArgsConfig(args) {
  return jsutils_keyValMap.keyValMap(args, (arg => arg.name), (arg => ({
    description: arg.description,
    type: arg.type,
    defaultValue: arg.defaultValue,
    deprecationReason: arg.deprecationReason,
    extensions: arg.extensions,
    astNode: arg.astNode
  })));
}

class GraphQLInterfaceType {
  constructor(config) {
    var _config$extensionASTN3;
    this.name = type_assertName.assertName(config.name);
    this.description = config.description;
    this.resolveType = config.resolveType;
    this.extensions = jsutils_toObjMap.toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = null !== (_config$extensionASTN3 = config.extensionASTNodes) && void 0 !== _config$extensionASTN3 ? _config$extensionASTN3 : [];
    this._fields = defineFieldMap.bind(void 0, config);
    this._interfaces = defineInterfaces.bind(void 0, config);
  }
  getFields() {
    if ("function" == typeof this._fields) {
      this._fields = this._fields();
    }
    return this._fields;
  }
  getInterfaces() {
    if ("function" == typeof this._interfaces) {
      this._interfaces = this._interfaces();
    }
    return this._interfaces;
  }
  toConfig() {
    return {
      name: this.name,
      description: this.description,
      interfaces: this.getInterfaces(),
      fields: fieldsToFieldsConfig(this.getFields()),
      resolveType: this.resolveType,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes
    };
  }
  toString() {
    return this.name;
  }
  toJSON() {
    return this.toString();
  }
}

class GraphQLUnionType {
  constructor(config) {
    var _config$extensionASTN4;
    this.name = type_assertName.assertName(config.name);
    this.description = config.description;
    this.resolveType = config.resolveType;
    this.extensions = jsutils_toObjMap.toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = null !== (_config$extensionASTN4 = config.extensionASTNodes) && void 0 !== _config$extensionASTN4 ? _config$extensionASTN4 : [];
    this._types = defineTypes.bind(void 0, config);
  }
  getTypes() {
    if ("function" == typeof this._types) {
      this._types = this._types();
    }
    return this._types;
  }
  toConfig() {
    return {
      name: this.name,
      description: this.description,
      types: this.getTypes(),
      resolveType: this.resolveType,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes
    };
  }
  toString() {
    return this.name;
  }
  toJSON() {
    return this.toString();
  }
}

function defineTypes(config) {
  return resolveReadonlyArrayThunk(config.types);
}

class GraphQLEnumType {
  constructor(config) {
    var _config$extensionASTN5;
    this.name = type_assertName.assertName(config.name);
    this.description = config.description;
    this.extensions = jsutils_toObjMap.toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = null !== (_config$extensionASTN5 = config.extensionASTNodes) && void 0 !== _config$extensionASTN5 ? _config$extensionASTN5 : [];
    this._values = function defineEnumValues(typeName, valueMap) {
      return Object.entries(valueMap).map((([valueName, valueConfig]) => ({
        name: type_assertName.assertEnumValueName(valueName),
        description: valueConfig.description,
        value: void 0 !== valueConfig.value ? valueConfig.value : valueName,
        deprecationReason: valueConfig.deprecationReason,
        extensions: jsutils_toObjMap.toObjMap(valueConfig.extensions),
        astNode: valueConfig.astNode
      })));
    }(0, config.values);
    this._valueLookup = new Map(this._values.map((enumValue => [ enumValue.value, enumValue ])));
    this._nameLookup = jsutils_keyMap.keyMap(this._values, (value => value.name));
  }
  getValues() {
    return this._values;
  }
  getValue(name) {
    return this._nameLookup[name];
  }
  serialize(outputValue) {
    const enumValue = this._valueLookup.get(outputValue);
    if (void 0 === enumValue) {
      throw new shared.GraphQLError(`Enum "${this.name}" cannot represent value: ${jsutils_inspect.inspect(outputValue)}`);
    }
    return enumValue.name;
  }
  parseValue(inputValue) {
    if ("string" != typeof inputValue) {
      const valueStr = jsutils_inspect.inspect(inputValue);
      throw new shared.GraphQLError(`Enum "${this.name}" cannot represent non-string value: ${valueStr}.` + didYouMeanEnumValue(this, valueStr));
    }
    const enumValue = this.getValue(inputValue);
    if (null == enumValue) {
      throw new shared.GraphQLError(`Value "${inputValue}" does not exist in "${this.name}" enum.` + didYouMeanEnumValue(this, inputValue));
    }
    return enumValue.value;
  }
  parseLiteral(valueNode, _variables) {
    if (valueNode.kind !== shared.e.ENUM) {
      const valueStr = shared.print(valueNode);
      throw new shared.GraphQLError(`Enum "${this.name}" cannot represent non-enum value: ${valueStr}.` + didYouMeanEnumValue(this, valueStr), {
        nodes: valueNode
      });
    }
    const enumValue = this.getValue(valueNode.value);
    if (null == enumValue) {
      const valueStr = shared.print(valueNode);
      throw new shared.GraphQLError(`Value "${valueStr}" does not exist in "${this.name}" enum.` + didYouMeanEnumValue(this, valueStr), {
        nodes: valueNode
      });
    }
    return enumValue.value;
  }
  toConfig() {
    const values = jsutils_keyValMap.keyValMap(this.getValues(), (value => value.name), (value => ({
      description: value.description,
      value: value.value,
      deprecationReason: value.deprecationReason,
      extensions: value.extensions,
      astNode: value.astNode
    })));
    return {
      name: this.name,
      description: this.description,
      values,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes
    };
  }
  toString() {
    return this.name;
  }
  toJSON() {
    return this.toString();
  }
}

function didYouMeanEnumValue(enumType, unknownValueStr) {
  const allNames = enumType.getValues().map((value => value.name));
  const suggestedValues = jsutils_suggestionList.suggestionList(unknownValueStr, allNames);
  return jsutils_didYouMean.didYouMean("the enum value", suggestedValues);
}

class GraphQLInputObjectType {
  constructor(config) {
    var _config$extensionASTN6;
    this.name = type_assertName.assertName(config.name);
    this.description = config.description;
    this.extensions = jsutils_toObjMap.toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = null !== (_config$extensionASTN6 = config.extensionASTNodes) && void 0 !== _config$extensionASTN6 ? _config$extensionASTN6 : [];
    this._fields = defineInputFieldMap.bind(void 0, config);
  }
  getFields() {
    if ("function" == typeof this._fields) {
      this._fields = this._fields();
    }
    return this._fields;
  }
  toConfig() {
    const fields = jsutils_mapValue.mapValue(this.getFields(), (field => ({
      description: field.description,
      type: field.type,
      defaultValue: field.defaultValue,
      deprecationReason: field.deprecationReason,
      extensions: field.extensions,
      astNode: field.astNode
    })));
    return {
      name: this.name,
      description: this.description,
      fields,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes
    };
  }
  toString() {
    return this.name;
  }
  toJSON() {
    return this.toString();
  }
}

function defineInputFieldMap(config) {
  const fieldMap = resolveObjMapThunk(config.fields);
  return jsutils_mapValue.mapValue(fieldMap, ((fieldConfig, fieldName) => ({
    name: type_assertName.assertName(fieldName),
    description: fieldConfig.description,
    type: fieldConfig.type,
    defaultValue: fieldConfig.defaultValue,
    deprecationReason: fieldConfig.deprecationReason,
    extensions: jsutils_toObjMap.toObjMap(fieldConfig.extensions),
    astNode: fieldConfig.astNode
  })));
}

exports.GraphQLEnumType = GraphQLEnumType;

exports.GraphQLInputObjectType = GraphQLInputObjectType;

exports.GraphQLInterfaceType = GraphQLInterfaceType;

exports.GraphQLList = GraphQLList;

exports.GraphQLNonNull = GraphQLNonNull;

exports.GraphQLObjectType = GraphQLObjectType;

exports.GraphQLScalarType = GraphQLScalarType;

exports.GraphQLUnionType = GraphQLUnionType;

exports.argsToArgsConfig = argsToArgsConfig;

exports.assertAbstractType = function assertAbstractType(type) {
  if (!isAbstractType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL abstract type.`);
  }
  return type;
};

exports.assertCompositeType = function assertCompositeType(type) {
  if (!isCompositeType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL composite type.`);
  }
  return type;
};

exports.assertEnumType = function assertEnumType(type) {
  if (!isEnumType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL Enum type.`);
  }
  return type;
};

exports.assertInputObjectType = function assertInputObjectType(type) {
  if (!isInputObjectType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL Input Object type.`);
  }
  return type;
};

exports.assertInputType = function assertInputType(type) {
  if (!isInputType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL input type.`);
  }
  return type;
};

exports.assertInterfaceType = function assertInterfaceType(type) {
  if (!isInterfaceType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL Interface type.`);
  }
  return type;
};

exports.assertLeafType = function assertLeafType(type) {
  if (!isLeafType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL leaf type.`);
  }
  return type;
};

exports.assertListType = function assertListType(type) {
  if (!isListType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL List type.`);
  }
  return type;
};

exports.assertNamedType = function assertNamedType(type) {
  if (!isNamedType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL named type.`);
  }
  return type;
};

exports.assertNonNullType = function assertNonNullType(type) {
  if (!isNonNullType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL Non-Null type.`);
  }
  return type;
};

exports.assertNullableType = function assertNullableType(type) {
  if (!isNullableType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL nullable type.`);
  }
  return type;
};

exports.assertObjectType = function assertObjectType(type) {
  if (!isObjectType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL Object type.`);
  }
  return type;
};

exports.assertOutputType = function assertOutputType(type) {
  if (!isOutputType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL output type.`);
  }
  return type;
};

exports.assertScalarType = function assertScalarType(type) {
  if (!isScalarType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL Scalar type.`);
  }
  return type;
};

exports.assertType = function assertType(type) {
  if (!isType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL type.`);
  }
  return type;
};

exports.assertUnionType = function assertUnionType(type) {
  if (!isUnionType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL Union type.`);
  }
  return type;
};

exports.assertWrappingType = function assertWrappingType(type) {
  if (!isWrappingType(type)) {
    throw new Error(`Expected ${jsutils_inspect.inspect(type)} to be a GraphQL wrapping type.`);
  }
  return type;
};

exports.defineArguments = defineArguments;

exports.getNamedType = function getNamedType(type) {
  if (type) {
    let unwrappedType = type;
    while (isWrappingType(unwrappedType)) {
      unwrappedType = unwrappedType.ofType;
    }
    return unwrappedType;
  }
};

exports.getNullableType = function getNullableType(type) {
  if (type) {
    return isNonNullType(type) ? type.ofType : type;
  }
};

exports.isAbstractType = isAbstractType;

exports.isCompositeType = isCompositeType;

exports.isEnumType = isEnumType;

exports.isInputObjectType = isInputObjectType;

exports.isInputType = isInputType;

exports.isInterfaceType = isInterfaceType;

exports.isLeafType = isLeafType;

exports.isListType = isListType;

exports.isNamedType = isNamedType;

exports.isNonNullType = isNonNullType;

exports.isNullableType = isNullableType;

exports.isObjectType = isObjectType;

exports.isOutputType = isOutputType;

exports.isRequiredArgument = function isRequiredArgument(arg) {
  return isNonNullType(arg.type) && void 0 === arg.defaultValue;
};

exports.isRequiredInputField = function isRequiredInputField(field) {
  return isNonNullType(field.type) && void 0 === field.defaultValue;
};

exports.isScalarType = isScalarType;

exports.isType = isType;

exports.isUnionType = isUnionType;

exports.isWrappingType = isWrappingType;
