var jsutils_invariant = require("../jsutils/invariant.js");

var language_directiveLocation = require("../language/directiveLocation.js");

var utilities_astFromValue = require("../utilities/astFromValue.js");

var type_definition = require("./definition.js");

var type_scalars = require("./scalars.js");

var shared = require("../shared.js");

const __Schema = new type_definition.GraphQLObjectType({
  name: "__Schema",
  description: "A GraphQL Schema defines the capabilities of a GraphQL server. It exposes all available types and directives on the server, as well as the entry points for query, mutation, and subscription operations.",
  fields: () => ({
    description: {
      type: type_scalars.GraphQLString,
      resolve: schema => schema.description
    },
    types: {
      description: "A list of all types supported by this server.",
      type: new type_definition.GraphQLNonNull(new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__Type))),
      resolve: schema => Object.values(schema.getTypeMap())
    },
    queryType: {
      description: "The type that query operations will be rooted at.",
      type: new type_definition.GraphQLNonNull(__Type),
      resolve: schema => schema.getQueryType()
    },
    mutationType: {
      description: "If this server supports mutation, the type that mutation operations will be rooted at.",
      type: __Type,
      resolve: schema => schema.getMutationType()
    },
    subscriptionType: {
      description: "If this server support subscription, the type that subscription operations will be rooted at.",
      type: __Type,
      resolve: schema => schema.getSubscriptionType()
    },
    directives: {
      description: "A list of all directives supported by this server.",
      type: new type_definition.GraphQLNonNull(new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__Directive))),
      resolve: schema => schema.getDirectives()
    }
  })
});

const __Directive = new type_definition.GraphQLObjectType({
  name: "__Directive",
  description: "A Directive provides a way to describe alternate runtime execution and type validation behavior in a GraphQL document.\n\nIn some cases, you need to provide options to alter GraphQL's execution behavior in ways field arguments will not suffice, such as conditionally including or skipping a field. Directives provide this by describing additional information to the executor.",
  fields: () => ({
    name: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLString),
      resolve: directive => directive.name
    },
    description: {
      type: type_scalars.GraphQLString,
      resolve: directive => directive.description
    },
    isRepeatable: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLBoolean),
      resolve: directive => directive.isRepeatable
    },
    locations: {
      type: new type_definition.GraphQLNonNull(new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__DirectiveLocation))),
      resolve: directive => directive.locations
    },
    args: {
      type: new type_definition.GraphQLNonNull(new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__InputValue))),
      args: {
        includeDeprecated: {
          type: type_scalars.GraphQLBoolean,
          defaultValue: !1
        }
      },
      resolve: (field, {includeDeprecated}) => includeDeprecated ? field.args : field.args.filter((arg => null == arg.deprecationReason))
    }
  })
});

const __DirectiveLocation = new type_definition.GraphQLEnumType({
  name: "__DirectiveLocation",
  description: "A Directive can be adjacent to many parts of the GraphQL language, a __DirectiveLocation describes one such possible adjacencies.",
  values: {
    QUERY: {
      value: language_directiveLocation.DirectiveLocation.QUERY,
      description: "Location adjacent to a query operation."
    },
    MUTATION: {
      value: language_directiveLocation.DirectiveLocation.MUTATION,
      description: "Location adjacent to a mutation operation."
    },
    SUBSCRIPTION: {
      value: language_directiveLocation.DirectiveLocation.SUBSCRIPTION,
      description: "Location adjacent to a subscription operation."
    },
    FIELD: {
      value: language_directiveLocation.DirectiveLocation.FIELD,
      description: "Location adjacent to a field."
    },
    FRAGMENT_DEFINITION: {
      value: language_directiveLocation.DirectiveLocation.FRAGMENT_DEFINITION,
      description: "Location adjacent to a fragment definition."
    },
    FRAGMENT_SPREAD: {
      value: language_directiveLocation.DirectiveLocation.FRAGMENT_SPREAD,
      description: "Location adjacent to a fragment spread."
    },
    INLINE_FRAGMENT: {
      value: language_directiveLocation.DirectiveLocation.INLINE_FRAGMENT,
      description: "Location adjacent to an inline fragment."
    },
    VARIABLE_DEFINITION: {
      value: language_directiveLocation.DirectiveLocation.VARIABLE_DEFINITION,
      description: "Location adjacent to a variable definition."
    },
    SCHEMA: {
      value: language_directiveLocation.DirectiveLocation.SCHEMA,
      description: "Location adjacent to a schema definition."
    },
    SCALAR: {
      value: language_directiveLocation.DirectiveLocation.SCALAR,
      description: "Location adjacent to a scalar definition."
    },
    OBJECT: {
      value: language_directiveLocation.DirectiveLocation.OBJECT,
      description: "Location adjacent to an object type definition."
    },
    FIELD_DEFINITION: {
      value: language_directiveLocation.DirectiveLocation.FIELD_DEFINITION,
      description: "Location adjacent to a field definition."
    },
    ARGUMENT_DEFINITION: {
      value: language_directiveLocation.DirectiveLocation.ARGUMENT_DEFINITION,
      description: "Location adjacent to an argument definition."
    },
    INTERFACE: {
      value: language_directiveLocation.DirectiveLocation.INTERFACE,
      description: "Location adjacent to an interface definition."
    },
    UNION: {
      value: language_directiveLocation.DirectiveLocation.UNION,
      description: "Location adjacent to a union definition."
    },
    ENUM: {
      value: language_directiveLocation.DirectiveLocation.ENUM,
      description: "Location adjacent to an enum definition."
    },
    ENUM_VALUE: {
      value: language_directiveLocation.DirectiveLocation.ENUM_VALUE,
      description: "Location adjacent to an enum value definition."
    },
    INPUT_OBJECT: {
      value: language_directiveLocation.DirectiveLocation.INPUT_OBJECT,
      description: "Location adjacent to an input object type definition."
    },
    INPUT_FIELD_DEFINITION: {
      value: language_directiveLocation.DirectiveLocation.INPUT_FIELD_DEFINITION,
      description: "Location adjacent to an input object field definition."
    }
  }
});

const __Type = new type_definition.GraphQLObjectType({
  name: "__Type",
  description: "The fundamental unit of any GraphQL Schema is the type. There are many kinds of types in GraphQL as represented by the `__TypeKind` enum.\n\nDepending on the kind of a type, certain fields describe information about that type. Scalar types provide no information beyond a name, description and optional `specifiedByURL`, while Enum types provide their values. Object and Interface types provide the fields they describe. Abstract types, Union and Interface, provide the Object types possible at runtime. List and NonNull types compose other types.",
  fields: () => ({
    kind: {
      type: new type_definition.GraphQLNonNull(__TypeKind),
      resolve(type) {
        if (type_definition.isScalarType(type)) {
          return exports.TypeKind.SCALAR;
        }
        if (type_definition.isObjectType(type)) {
          return exports.TypeKind.OBJECT;
        }
        if (type_definition.isInterfaceType(type)) {
          return exports.TypeKind.INTERFACE;
        }
        if (type_definition.isUnionType(type)) {
          return exports.TypeKind.UNION;
        }
        if (type_definition.isEnumType(type)) {
          return exports.TypeKind.ENUM;
        }
        if (type_definition.isInputObjectType(type)) {
          return exports.TypeKind.INPUT_OBJECT;
        }
        if (type_definition.isListType(type)) {
          return exports.TypeKind.LIST;
        }
        if (type_definition.isNonNullType(type)) {
          return exports.TypeKind.NON_NULL;
        }
        jsutils_invariant.invariant(!1);
      }
    },
    name: {
      type: type_scalars.GraphQLString,
      resolve: type => "name" in type ? type.name : void 0
    },
    description: {
      type: type_scalars.GraphQLString,
      resolve: type => "description" in type ? type.description : void 0
    },
    specifiedByURL: {
      type: type_scalars.GraphQLString,
      resolve: obj => "specifiedByURL" in obj ? obj.specifiedByURL : void 0
    },
    fields: {
      type: new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__Field)),
      args: {
        includeDeprecated: {
          type: type_scalars.GraphQLBoolean,
          defaultValue: !1
        }
      },
      resolve(type, {includeDeprecated}) {
        if (type_definition.isObjectType(type) || type_definition.isInterfaceType(type)) {
          const fields = Object.values(type.getFields());
          return includeDeprecated ? fields : fields.filter((field => null == field.deprecationReason));
        }
      }
    },
    interfaces: {
      type: new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__Type)),
      resolve(type) {
        if (type_definition.isObjectType(type) || type_definition.isInterfaceType(type)) {
          return type.getInterfaces();
        }
      }
    },
    possibleTypes: {
      type: new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__Type)),
      resolve(type, _args, _context, {schema}) {
        if (type_definition.isAbstractType(type)) {
          return schema.getPossibleTypes(type);
        }
      }
    },
    enumValues: {
      type: new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__EnumValue)),
      args: {
        includeDeprecated: {
          type: type_scalars.GraphQLBoolean,
          defaultValue: !1
        }
      },
      resolve(type, {includeDeprecated}) {
        if (type_definition.isEnumType(type)) {
          const values = type.getValues();
          return includeDeprecated ? values : values.filter((field => null == field.deprecationReason));
        }
      }
    },
    inputFields: {
      type: new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__InputValue)),
      args: {
        includeDeprecated: {
          type: type_scalars.GraphQLBoolean,
          defaultValue: !1
        }
      },
      resolve(type, {includeDeprecated}) {
        if (type_definition.isInputObjectType(type)) {
          const values = Object.values(type.getFields());
          return includeDeprecated ? values : values.filter((field => null == field.deprecationReason));
        }
      }
    },
    ofType: {
      type: __Type,
      resolve: type => "ofType" in type ? type.ofType : void 0
    }
  })
});

const __Field = new type_definition.GraphQLObjectType({
  name: "__Field",
  description: "Object and Interface types are described by a list of Fields, each of which has a name, potentially a list of arguments, and a return type.",
  fields: () => ({
    name: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLString),
      resolve: field => field.name
    },
    description: {
      type: type_scalars.GraphQLString,
      resolve: field => field.description
    },
    args: {
      type: new type_definition.GraphQLNonNull(new type_definition.GraphQLList(new type_definition.GraphQLNonNull(__InputValue))),
      args: {
        includeDeprecated: {
          type: type_scalars.GraphQLBoolean,
          defaultValue: !1
        }
      },
      resolve: (field, {includeDeprecated}) => includeDeprecated ? field.args : field.args.filter((arg => null == arg.deprecationReason))
    },
    type: {
      type: new type_definition.GraphQLNonNull(__Type),
      resolve: field => field.type
    },
    isDeprecated: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLBoolean),
      resolve: field => null != field.deprecationReason
    },
    deprecationReason: {
      type: type_scalars.GraphQLString,
      resolve: field => field.deprecationReason
    }
  })
});

const __InputValue = new type_definition.GraphQLObjectType({
  name: "__InputValue",
  description: "Arguments provided to Fields or Directives and the input fields of an InputObject are represented as Input Values which describe their type and optionally a default value.",
  fields: () => ({
    name: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLString),
      resolve: inputValue => inputValue.name
    },
    description: {
      type: type_scalars.GraphQLString,
      resolve: inputValue => inputValue.description
    },
    type: {
      type: new type_definition.GraphQLNonNull(__Type),
      resolve: inputValue => inputValue.type
    },
    defaultValue: {
      type: type_scalars.GraphQLString,
      description: "A GraphQL-formatted string representing the default value for this input value.",
      resolve(inputValue) {
        const {type, defaultValue} = inputValue;
        const valueAST = utilities_astFromValue.astFromValue(defaultValue, type);
        return valueAST ? shared.print(valueAST) : null;
      }
    },
    isDeprecated: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLBoolean),
      resolve: field => null != field.deprecationReason
    },
    deprecationReason: {
      type: type_scalars.GraphQLString,
      resolve: obj => obj.deprecationReason
    }
  })
});

const __EnumValue = new type_definition.GraphQLObjectType({
  name: "__EnumValue",
  description: "One possible value for a given Enum. Enum values are unique values, not a placeholder for a string or numeric value. However an Enum value is returned in a JSON response as a string.",
  fields: () => ({
    name: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLString),
      resolve: enumValue => enumValue.name
    },
    description: {
      type: type_scalars.GraphQLString,
      resolve: enumValue => enumValue.description
    },
    isDeprecated: {
      type: new type_definition.GraphQLNonNull(type_scalars.GraphQLBoolean),
      resolve: enumValue => null != enumValue.deprecationReason
    },
    deprecationReason: {
      type: type_scalars.GraphQLString,
      resolve: enumValue => enumValue.deprecationReason
    }
  })
});

exports.TypeKind = void 0;

!function(TypeKind) {
  TypeKind.SCALAR = "SCALAR";
  TypeKind.OBJECT = "OBJECT";
  TypeKind.INTERFACE = "INTERFACE";
  TypeKind.UNION = "UNION";
  TypeKind.ENUM = "ENUM";
  TypeKind.INPUT_OBJECT = "INPUT_OBJECT";
  TypeKind.LIST = "LIST";
  TypeKind.NON_NULL = "NON_NULL";
}(exports.TypeKind || (exports.TypeKind = {}));

const __TypeKind = new type_definition.GraphQLEnumType({
  name: "__TypeKind",
  description: "An enum describing what kind of type a given `__Type` is.",
  values: {
    SCALAR: {
      value: exports.TypeKind.SCALAR,
      description: "Indicates this type is a scalar."
    },
    OBJECT: {
      value: exports.TypeKind.OBJECT,
      description: "Indicates this type is an object. `fields` and `interfaces` are valid fields."
    },
    INTERFACE: {
      value: exports.TypeKind.INTERFACE,
      description: "Indicates this type is an interface. `fields`, `interfaces`, and `possibleTypes` are valid fields."
    },
    UNION: {
      value: exports.TypeKind.UNION,
      description: "Indicates this type is a union. `possibleTypes` is a valid field."
    },
    ENUM: {
      value: exports.TypeKind.ENUM,
      description: "Indicates this type is an enum. `enumValues` is a valid field."
    },
    INPUT_OBJECT: {
      value: exports.TypeKind.INPUT_OBJECT,
      description: "Indicates this type is an input object. `inputFields` is a valid field."
    },
    LIST: {
      value: exports.TypeKind.LIST,
      description: "Indicates this type is a list. `ofType` is a valid field."
    },
    NON_NULL: {
      value: exports.TypeKind.NON_NULL,
      description: "Indicates this type is a non-null. `ofType` is a valid field."
    }
  }
});

const SchemaMetaFieldDef = {
  name: "__schema",
  type: new type_definition.GraphQLNonNull(__Schema),
  description: "Access the current type schema of this server.",
  args: [],
  resolve: (_source, _args, _context, {schema}) => schema,
  deprecationReason: void 0,
  extensions: Object.create(null),
  astNode: void 0
};

const TypeMetaFieldDef = {
  name: "__type",
  type: __Type,
  description: "Request the type information of a single type.",
  args: [ {
    name: "name",
    description: void 0,
    type: new type_definition.GraphQLNonNull(type_scalars.GraphQLString),
    defaultValue: void 0,
    deprecationReason: void 0,
    extensions: Object.create(null),
    astNode: void 0
  } ],
  resolve: (_source, {name}, _context, {schema}) => schema.getType(name),
  deprecationReason: void 0,
  extensions: Object.create(null),
  astNode: void 0
};

const TypeNameMetaFieldDef = {
  name: "__typename",
  type: new type_definition.GraphQLNonNull(type_scalars.GraphQLString),
  description: "The name of the current Object type at runtime.",
  args: [],
  resolve: (_source, _args, _context, {parentType}) => parentType.name,
  deprecationReason: void 0,
  extensions: Object.create(null),
  astNode: void 0
};

const introspectionTypes = [ __Schema, __Directive, __DirectiveLocation, __Type, __Field, __InputValue, __EnumValue, __TypeKind ];

exports.SchemaMetaFieldDef = SchemaMetaFieldDef;

exports.TypeMetaFieldDef = TypeMetaFieldDef;

exports.TypeNameMetaFieldDef = TypeNameMetaFieldDef;

exports.__Directive = __Directive;

exports.__DirectiveLocation = __DirectiveLocation;

exports.__EnumValue = __EnumValue;

exports.__Field = __Field;

exports.__InputValue = __InputValue;

exports.__Schema = __Schema;

exports.__Type = __Type;

exports.__TypeKind = __TypeKind;

exports.introspectionTypes = introspectionTypes;

exports.isIntrospectionType = function isIntrospectionType(type) {
  return introspectionTypes.some((({name}) => type.name === name));
};
