import { isExecutableDefinitionNode } from "../../language/predicates.mjs";

import { GraphQLError, e } from "../../shared.mjs";

function ExecutableDefinitionsRule(context) {
  return {
    Document(node) {
      for (const definition of node.definitions) {
        if (!isExecutableDefinitionNode(definition)) {
          context.reportError(new GraphQLError(`The ${definition.kind === e.SCHEMA_DEFINITION || definition.kind === e.SCHEMA_EXTENSION ? "schema" : '"' + definition.name.value + '"'} definition is not executable.`, {
            nodes: definition
          }));
        }
      }
      return !1;
    }
  };
}

export { ExecutableDefinitionsRule };
