import { didYouMean } from "../../jsutils/didYouMean.mjs";

import { suggestionList } from "../../jsutils/suggestionList.mjs";

import { isTypeDefinitionNode, isTypeSystemDefinitionNode, isTypeSystemExtensionNode } from "../../language/predicates.mjs";

import { introspectionTypes } from "../../type/introspection.mjs";

import { specifiedScalarTypes } from "../../type/scalars.mjs";

import { GraphQLError } from "../../shared.mjs";

function KnownTypeNamesRule(context) {
  const schema = context.getSchema();
  const existingTypesMap = schema ? schema.getTypeMap() : Object.create(null);
  const definedTypes = Object.create(null);
  for (const def of context.getDocument().definitions) {
    if (isTypeDefinitionNode(def)) {
      definedTypes[def.name.value] = !0;
    }
  }
  const typeNames = [ ...Object.keys(existingTypesMap), ...Object.keys(definedTypes) ];
  return {
    NamedType(node, _1, parent, _2, ancestors) {
      const typeName = node.name.value;
      if (!existingTypesMap[typeName] && !definedTypes[typeName]) {
        var _ancestors$;
        const definitionNode = null !== (_ancestors$ = ancestors[2]) && void 0 !== _ancestors$ ? _ancestors$ : parent;
        const isSDL = null != definitionNode && function isSDLNode(value) {
          return "kind" in value && (isTypeSystemDefinitionNode(value) || isTypeSystemExtensionNode(value));
        }(definitionNode);
        if (isSDL && standardTypeNames.includes(typeName)) {
          return;
        }
        const suggestedTypes = suggestionList(typeName, isSDL ? standardTypeNames.concat(typeNames) : typeNames);
        context.reportError(new GraphQLError(`Unknown type "${typeName}".` + didYouMean(suggestedTypes), {
          nodes: node
        }));
      }
    }
  };
}

const standardTypeNames = [ ...specifiedScalarTypes, ...introspectionTypes ].map((type => type.name));

export { KnownTypeNamesRule };
