var shared = require("../../shared.js");

exports.NoUndefinedVariablesRule = function NoUndefinedVariablesRule(context) {
  let variableNameDefined = Object.create(null);
  return {
    OperationDefinition: {
      enter() {
        variableNameDefined = Object.create(null);
      },
      leave(operation) {
        const usages = context.getRecursiveVariableUsages(operation);
        for (const {node} of usages) {
          const varName = node.name.value;
          if (!0 !== variableNameDefined[varName]) {
            context.reportError(new shared.GraphQLError(operation.name ? `Variable "$${varName}" is not defined by operation "${operation.name.value}".` : `Variable "$${varName}" is not defined.`, {
              nodes: [ node, operation ]
            }));
          }
        }
      }
    },
    VariableDefinition(node) {
      variableNameDefined[node.variable.name.value] = !0;
    }
  };
};
