var shared = require("../../shared.js");

exports.UniqueDirectiveNamesRule = function UniqueDirectiveNamesRule(context) {
  const knownDirectiveNames = Object.create(null);
  const schema = context.getSchema();
  return {
    DirectiveDefinition(node) {
      const directiveName = node.name.value;
      if (null != schema && schema.getDirective(directiveName)) {
        context.reportError(new shared.GraphQLError(`Directive "@${directiveName}" already exists in the schema. It cannot be redefined.`, {
          nodes: node.name
        }));
        return;
      }
      if (knownDirectiveNames[directiveName]) {
        context.reportError(new shared.GraphQLError(`There can be only one directive named "@${directiveName}".`, {
          nodes: [ knownDirectiveNames[directiveName], node.name ]
        }));
      } else {
        knownDirectiveNames[directiveName] = node.name;
      }
      return !1;
    }
  };
};
