var type_definition = require("../../type/definition.js");

var shared = require("../../shared.js");

exports.UniqueEnumValueNamesRule = function UniqueEnumValueNamesRule(context) {
  const schema = context.getSchema();
  const existingTypeMap = schema ? schema.getTypeMap() : Object.create(null);
  const knownValueNames = Object.create(null);
  return {
    EnumTypeDefinition: checkValueUniqueness,
    EnumTypeExtension: checkValueUniqueness
  };
  function checkValueUniqueness(node) {
    var _node$values;
    const typeName = node.name.value;
    if (!knownValueNames[typeName]) {
      knownValueNames[typeName] = Object.create(null);
    }
    const valueNodes = null !== (_node$values = node.values) && void 0 !== _node$values ? _node$values : [];
    const valueNames = knownValueNames[typeName];
    for (const valueDef of valueNodes) {
      const valueName = valueDef.name.value;
      const existingType = existingTypeMap[typeName];
      if (type_definition.isEnumType(existingType) && existingType.getValue(valueName)) {
        context.reportError(new shared.GraphQLError(`Enum value "${typeName}.${valueName}" already exists in the schema. It cannot also be defined in this type extension.`, {
          nodes: valueDef.name
        }));
      } else if (valueNames[valueName]) {
        context.reportError(new shared.GraphQLError(`Enum value "${typeName}.${valueName}" can only be defined once.`, {
          nodes: [ valueNames[valueName], valueDef.name ]
        }));
      } else {
        valueNames[valueName] = valueDef.name;
      }
    }
    return !1;
  }
};
