import { inspect } from "../../jsutils/inspect.mjs";

import { isNonNullType } from "../../type/definition.mjs";

import { isTypeSubTypeOf } from "../../utilities/typeComparators.mjs";

import { typeFromAST } from "../../utilities/typeFromAST.mjs";

import { GraphQLError, e } from "../../shared.mjs";

function VariablesInAllowedPositionRule(context) {
  let varDefMap = Object.create(null);
  return {
    OperationDefinition: {
      enter() {
        varDefMap = Object.create(null);
      },
      leave(operation) {
        const usages = context.getRecursiveVariableUsages(operation);
        for (const {node, type, defaultValue} of usages) {
          const varName = node.name.value;
          const varDef = varDefMap[varName];
          if (varDef && type) {
            const schema = context.getSchema();
            const varType = typeFromAST(schema, varDef.type);
            if (varType && !allowedVariableUsage(schema, varType, varDef.defaultValue, type, defaultValue)) {
              const varTypeStr = inspect(varType);
              const typeStr = inspect(type);
              context.reportError(new GraphQLError(`Variable "$${varName}" of type "${varTypeStr}" used in position expecting type "${typeStr}".`, {
                nodes: [ varDef, node ]
              }));
            }
          }
        }
      }
    },
    VariableDefinition(node) {
      varDefMap[node.variable.name.value] = node;
    }
  };
}

function allowedVariableUsage(schema, varType, varDefaultValue, locationType, locationDefaultValue) {
  if (isNonNullType(locationType) && !isNonNullType(varType)) {
    if (!(null != varDefaultValue && varDefaultValue.kind !== e.NULL) && !(void 0 !== locationDefaultValue)) {
      return !1;
    }
    return isTypeSubTypeOf(schema, varType, locationType.ofType);
  }
  return isTypeSubTypeOf(schema, varType, locationType);
}

export { VariablesInAllowedPositionRule };
